'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CALL_API = exports.SCHEMAS = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _normalizr = require('normalizr');

require('isomorphic-fetch');

var _constants = require('./constants');

var _response = require('./response');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* Define schemas used for Api responses
 */
var schemaOpts = { idAttribute: '_id' };
var sessionSchema = new _normalizr.Schema('sessions', schemaOpts);
var recipeSchema = new _normalizr.Schema('recipes', schemaOpts);
var mealSchema = new _normalizr.Schema('meals', schemaOpts);
var userSchema = new _normalizr.Schema('users', schemaOpts);
var foodSchema = new _normalizr.Schema('foods', schemaOpts);
var ingredientFoodsSchema = new _normalizr.Schema('ingredientFoods', schemaOpts);
var ingredientRecipesSchema = new _normalizr.Schema('ingredientRecipes', schemaOpts);

sessionSchema.define({
  user: userSchema
});

foodSchema.define({
  author: userSchema
});

ingredientFoodsSchema.define({
  food: foodSchema
});

recipeSchema.define({
  author: userSchema,
  ingredientFoods: (0, _normalizr.arrayOf)(ingredientFoodsSchema)
});

ingredientRecipesSchema.define({
  recipe: recipeSchema
});

mealSchema.define({
  author: userSchema,
  ingredientFoods: (0, _normalizr.arrayOf)(ingredientFoodsSchema),
  ingredientRecipes: (0, _normalizr.arrayOf)(ingredientRecipesSchema)
});

/* Export available schemas corresponding for every
 * Api response (used by /actions).
 */
var SCHEMAS = exports.SCHEMAS = {
  SESSION: sessionSchema,

  USER: userSchema,
  USERS: (0, _normalizr.arrayOf)(userSchema),

  MEAL: mealSchema,
  MEALS: (0, _normalizr.arrayOf)(mealSchema),

  RECIPE: recipeSchema,
  RECIPES: (0, _normalizr.arrayOf)(recipeSchema),

  FOOD: foodSchema,
  FOODS: (0, _normalizr.arrayOf)(foodSchema)
};

/* Special symbol that signals this middleware to be
 * called in the redux middleware chain (used by /actions).
 */
var CALL_API = exports.CALL_API = 'Call API';

function assertValidCall(_ref) {
  var query = _ref.query,
      body = _ref.body,
      method = _ref.method,
      endpoint = _ref.endpoint,
      schema = _ref.schema,
      types = _ref.types;

  if (query && body) {
    throw new Error('Do not call with both query and body');
  }
  if (!method) {
    throw new Error('Specify a method for api call.');
  }
  if (['GET', 'POST', 'PUT', 'DELETE'].indexOf(method) === -1) {
    throw new Error('Method must be one of {GET, POST, PUT, DELETE}');
  }
  if (typeof endpoint !== 'string') {
    throw new Error('Specify a string endpoint URL.');
  }
  if (!schema) {
    throw new Error('Specify one of the exported Schemas.');
  }
  if (!Array.isArray(types) || types.length !== 4) {
    throw new Error('Expected an array of four action types.');
  }
  if (!types.every(function (type) {
    return typeof type === 'string';
  })) {
    throw new Error('Expected action types to be strings.');
  }
}

/* Middleware that interprets calls with CALL_API.
 */

exports.default = function (store) {
  return function (next) {
    return function (action) {
      var callAPI = action[CALL_API];
      if (typeof callAPI === 'undefined') {
        return next(action);
      }

      var token = callAPI.token,
          query = callAPI.query,
          body = callAPI.body,
          method = callAPI.method,
          endpoint = callAPI.endpoint,
          schema = callAPI.schema,
          types = callAPI.types;


      assertValidCall(callAPI);

      var url = _constants.API_ROOT + '/' + endpoint;

      var _types = _slicedToArray(types, 4),
          reqType = _types[0],
          okType = _types[1],
          cancelType = _types[2],
          errType = _types[3];

      var options = {
        credentials: 'include',
        method: method,
        headers: {
          'Accept': 'application/json',
          'Content-Type': 'application/json'
        }
      };

      if (token) {
        options.headers.Authorization = token;
      }

      if (body) {
        options.body = JSON.stringify(body);
      }

      if (query && !(0, _isEmpty2.default)(query)) {
        url = url + '?' + (0, _response.makeQueryString)(query);
      }

      next({ type: reqType });
      return fetch(url, options).then(_response.parseJSON).then((0, _response.apiHandler)(function (json) {
        next({
          type: okType,
          response: (0, _normalizr.normalize)(json, schema)
        });
      }, function (error, mongoose) {
        next({ type: cancelType });
        next({
          type: errType,
          error: error,
          mongoose: mongoose
        });
      }));
    };
  };
};